#region Using declarations
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Windows.Media;
using System.Xml.Serialization;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators.Devside
{
    public class SMI : Indicator
    {
        private List<double> _savedHighs = new List<double>();
        private List<double> _savedLows = new List<double>();
        private double _weight1, _weight2;
        private double max, min;
        private double m, d;
        private double dEMA, ds;
        private double _lastDEMA, _lastDs;
        private double diffEMA, dhl;
        private double _lastDiffEMA, _lastDhl;
        private double _lastSignalLine;

        protected override void OnStateChange()
        {

            if (State == State.SetDefaults)
            {
                Description = @"SMI";
                Name = "Stochastic Momentum Index";
                Calculate = Calculate.OnBarClose;
                IsOverlay = false;
                DisplayInDataBox = true;
                DrawOnPricePanel = true;
                DrawHorizontalGridLines = true;
                DrawVerticalGridLines = true;
                PaintPriceMarkers = true;
                ScaleJustification = NinjaTrader.Gui.Chart.ScaleJustification.Right;
                //Disable this property if your indicator requires custom values that cumulate with each new market data event. 
                //See Help Guide for additional information.
                IsSuspendedWhileInactive = true;
                Period = 14;
                BarsRequiredToPlot = 0;

                AddPlot(Brushes.Cyan, "SMI");
                AddPlot(Brushes.Red, "SignalLine");

                AddLine(Brushes.DodgerBlue, -50, NinjaTrader.Custom.Resource.NinjaScriptIndicatorLower);
                AddLine(Brushes.DodgerBlue, 50, NinjaTrader.Custom.Resource.NinjaScriptIndicatorUpper);
            }
            else if (State == State.Configure)
            {
                _weight1 = 2d / (1 + Period);
                _weight2 = 1 - _weight1;
            }
        }

        protected override void OnBarUpdate()
        {
            // Update High and Low values
            _savedHighs.Add(High[0]);
            if (_savedHighs.Count > Period) _savedHighs.RemoveAt(0);
            _savedLows.Add(Low[0]);
            if (_savedLows.Count > Period) _savedLows.RemoveAt(0);

            max = _savedHighs.Max();
            min = _savedLows.Min();

            // Calculate d
            m = (max + min) / 2;
            d = Close[0] - m;

            // Calculate ds
            dEMA = EMA(_lastDEMA, d);
            ds = EMA(_lastDs, dEMA);

            // Calculate dhl
            diffEMA = EMA(_lastDiffEMA, max - min);
            dhl = EMA(_lastDhl, diffEMA);

            Index[0] = dhl != 0 ? 100 * (ds / (0.5 * dhl)) : 0;
            SignalLine[0] = EMA(_lastSignalLine, Index[0]);

            _lastDEMA = dEMA;
            _lastDs = ds;
            _lastDiffEMA = diffEMA;
            _lastDhl = dhl;
            _lastSignalLine = SignalLine[0];
        }

        private double EMA(double? lastEMA, double value)
        {
            return (_weight1 * value) + (_weight2 * lastEMA) ?? value;
        }

        #region Properties
        [NinjaScriptProperty]
        [Range(1, int.MaxValue)]
        [Display(Name = "Period", Order = 1, GroupName = "Parameters")]
        public int Period
        { get; set; }

        [Browsable(false)]
        [XmlIgnore]
        public Series<double> Index
        {
            get { return Values[0]; }
        }

        [Browsable(false)]
        [XmlIgnore]
        public Series<double> SignalLine
        {
            get { return Values[1]; }
        }
        #endregion

    }
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private Devside.SMI[] cacheSMI;
		public Devside.SMI SMI(int period)
		{
			return SMI(Input, period);
		}

		public Devside.SMI SMI(ISeries<double> input, int period)
		{
			if (cacheSMI != null)
				for (int idx = 0; idx < cacheSMI.Length; idx++)
					if (cacheSMI[idx] != null && cacheSMI[idx].Period == period && cacheSMI[idx].EqualsInput(input))
						return cacheSMI[idx];
			return CacheIndicator<Devside.SMI>(new Devside.SMI(){ Period = period }, input, ref cacheSMI);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.Devside.SMI SMI(int period)
		{
			return indicator.SMI(Input, period);
		}

		public Indicators.Devside.SMI SMI(ISeries<double> input , int period)
		{
			return indicator.SMI(input, period);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.Devside.SMI SMI(int period)
		{
			return indicator.SMI(Input, period);
		}

		public Indicators.Devside.SMI SMI(ISeries<double> input , int period)
		{
			return indicator.SMI(input, period);
		}
	}
}

#endregion
